Option Explicit On

Imports SolidWorks.Interop.sldworks
Imports SolidWorks.Interop.swconst.swUserPreferenceDoubleValue_e   'swImageQualityShadedDeviation
Imports SolidWorks.Interop.swconst.swUserPreferenceToggle_e   'swImageQualityUseHighQualityEdgeSize
Imports SolidWorks.Interop.swconst.swUserPreferenceIntegerValue_e   'swImageQualityWireframeValue
Imports SolidWorks.Interop.swconst.swUserPreferenceOption_e   'swDetailingNoOptionSpecified
Imports System.Windows.Forms   'Application.DoEvents()

Partial Class SolidWorksMacro
    Public swApp As SldWorks

    Public Sub main()

        Dim swModel As SolidWorks.Interop.sldworks.ModelDoc2
        Dim swModelDocExt As SolidWorks.Interop.sldworks.ModelDocExtension
        Dim swStatus As SolidWorks.Interop.sldworks.Frame
        Dim bRet As Boolean

        ' check if Solidworks connected
        If swApp Is Nothing Then
            MsgBox("Sorry - Error Connecting to SolidWorks. Please Try Again. ", vbCritical, "Image Quality Optimizer")
            Exit Sub
        Else
            ' since SolidWorks is connected
            swModel = swApp.ActiveDoc
            swModelDocExt = swModel.Extension
            swStatus = swApp.Frame
        End If

        ' check if a model is open
        If swModel Is Nothing Then
            MsgBox("A SolidWorks file must be open.  ", vbCritical, "Image Quality Optimizer")
            Exit Sub
        End If

        ' / / / / / / / / / / create Time for Rebuild BEFORE Image Quality Optimizing \ \ \ \ \ \ \ \ \ \
        Dim startBefore As DateTime
        Dim stopBefore As DateTime
        Dim elapsedBefore As TimeSpan
        Dim sElapsedBefore As String

        ' start BEFORE elapsed timer
        startBefore = Now

        ' rebuild
        swStatus.SetStatusBarText(" Rebuilding BEFORE Optimizing")
        Application.DoEvents()
        ' bRet = swModel.ForceRebuild3(False)  ' Use to force a rebuild of the top-level assembly and all subassemblies (Caution - this may cause undesired results)
        ' bRet = swModel.ForceRebuild3(True)  ' Use to force a rebuild of the top-level assembly only (Caution - this may cause undesired results)
        bRet = swModel.ForceRebuild ' Use to force a rebuild (like Ctrl + Q) all of the features in the model whether or not they need a rebuild
        ' bRet = swModel.EditRebuild3 ' Use rebuild (like Ctrl + B) for only those features in the model that need a rebuild

        ' stop BEFORE elapsed timer
        stopBefore = Now
        elapsedBefore = stopBefore.Subtract(startBefore)
        sElapsedBefore = elapsedBefore.TotalSeconds.ToString("0.0000")
        ' / / / / / / / / / / End BEFORE Time \ \ \ \ \ \ \ \ \ \

        ' / / / / / / / / / / Set Image Quality \ \ \ \ \ \ \ \ \ \
        ' ******************************************************************************
        ' Optimize Image Quality settings - macro first recorded on 03/28/08 by sparkstd
        ' much of the credit for this goes to http://www.eng-tips.com/ -handleman
        ' ******************************************************************************

        ' create variables to retrieve current settings
        Dim dCurVal As Double
        Dim dMaxVal As Double
        Dim dMinVal As Double
        Dim dNewVal As Double
        Dim dMaxDesRes As Double
        Dim dMinDesRes As Double

        'For assemblies only; specifies whether to apply image quality settings to all part documents referenced by active document
        bRet = swModelDocExt.SetUserPreferenceToggle(swImageQualityApplyToAllReferencedPartDoc, swDetailingNoOptionSpecified, True)

        ' get the current, minimum and maximum values from the Image Quality Shaded Deviation Preferences
        ' load these values to variables "dCurVal", "dMinVal", and "dMaxVal"
        swModelDocExt.GetUserPreferenceDoubleValueRange(swImageQualityShadedDeviation, dCurVal, dMinVal, dMaxVal)

        ' calculate a New Value that sets the slider bar to approximately 1/3rd between Low and High resolution (Desired Resolution)
        dMaxDesRes = dMaxVal * 0.1005875 ' this constant is derived from empirical data
        dMinDesRes = dMinVal * 2.784475 ' this constant is derived from empirical data
        dNewVal = (dMaxDesRes + dMinDesRes) / 2

        ' Set new value for Shaded Deviation
        swModel.SetUserPreferenceDoubleValue(swImageQualityShadedDeviation, dNewVal)

        ' Set check box for "Optimize edge length"
        swModel.SetUserPreferenceToggle(swImageQualityUseHighQualityEdgeSize, False)

        ' Set new value for "Wireframe and high quality HLR/HLV resolution"
        swModel.SetUserPreferenceIntegerValue(swImageQualityWireframeValue, 5) ' this could be set lower, it's up to you

        ' / / / / / / / / / / create Time for Rebuild AFTER Image Quality Optimizing \ \ \ \ \ \ \ \ \ \
        Dim startAfter As DateTime
        Dim stopAfter As DateTime
        Dim elapsedAfter As TimeSpan
        Dim sElapsedAfter As String

        ' start AFTER elapsed timer
        startAfter = Now

        'rebuild
        swStatus.SetStatusBarText(" Rebuilding AFTER Optimizing")
        ' bRet = swModel.ForceRebuild3(False)  ' Use to force rebuild of the top-level assembly and all subassemblies (Caution - this may cause undesired results)
        ' bRet = swModel.ForceRebuild3(True)  ' Use to force rebuild of the top-level assembly only (Caution - this may cause undesired results)
        bRet = swModel.ForceRebuild ' Use to force rebuild (like Ctrl + Q) all of the features in the model whether or not they need a rebuild
        ' bRet = swModel.EditRebuild3 ' Use rebuild (like Ctrl + B) for only those features in the model that need a rebuild

        ' stop AFTER elapsed timer
        stopAfter = Now
        elapsedAfter = stopAfter.Subtract(startAfter)
        sElapsedAfter = elapsedAfter.TotalSeconds.ToString("0.0000")
        ' / / / / / / / / / / End AFTER Time \ \ \ \ \ \ \ \ \ \

        ' / / / / / / / / / / Display BEFORE and AFTER times \ \ \ \ \ \ \ \ \ \
        MsgBox(sElapsedBefore & " Seconds to Rebuild BEFORE Optimizing." & vbNewLine & _
        "----------------------------------------------------------------------------" & vbNewLine & _
        sElapsedAfter & " Seconds to Rebuild AFTER Optimizing.", vbInformation, "Image Quality Optimizer")

        ' / / / / / / / / / / Save \ \ \ \ \ \ \ \ \ \
        swStatus.SetStatusBarText(" Saving the Document")
        swModel.Save2(False) 'TRUE if you want to avoid error and warning dialogs, FALSE if you want the dialogs displayed to the user

        swStatus.SetStatusBarText(" Image Quality Optimizer - Done")
        Application.DoEvents()
        Beep()

    End Sub

End Class
